//EEPROM get
//By Holcsik Tams
// .:: MaSTeRFoXX ::.
//2009-09-05
//2009
//V1.0
//features
//


#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>


#define CE_H() PORTD|=(1<<PD6) //EEPROM chip enable
#define CE_L() PORTD&=~(1<<PD6)
#define OE_H() PORTD|=(1<<PD4) //EEPROM output enable
#define OE_L() PORTD&=~(1<<PD4)
#define WE_H() PORTD|=(1<<PD5) //EEPROM write enable
#define WE_L() PORTD&=~(1<<PD5)
//
#define LATCH1_PULSE()	PORTB|=(1<<PB2);  PORTB&=~(1<<PB2);   //L->H->L pulse    ->LOAD  ALS  8bit
#define LATCH2_PULSE()	PORTD|=(1<<PD7);  PORTD&=~(1<<PD7);	 //L->H->L pulse 	->LOAD	FELS 8bit
//
#define OUTPUT_MODE() DDRB|=(1<<PB0)|(1<<PB1); DDRC=0xFF;
#define INPUT_MODE() DDRB&=~(1<<PB0); DDRB&=~(1<<PB1); DDRC=0x00;


uint8_t usartchr=0;
uint16_t mem_size;
//uint16_t mem_addr;

void ser_byte(uint8_t send)
{
    while(!(UCSRA&(1<<UDRE)));
    UDR=send;
   
}

void endl(void)
{
	ser_byte(0x0a);
	ser_byte(0x0d);
}

void ser_decimal(uint8_t num)
{
    uint8_t szazas=0, tizes=0;
    while(num>=100)
    {
        num-=100;
        szazas++;
    }
    while(num>=10)
    {
        num-=10;
        tizes++;
    }
    ser_byte(szazas+48);
    ser_byte(tizes+48);
    ser_byte(num+48);
}
 
//bekapcsols utn felvillantja a ledeket
void startup_ledflash()
{
	for(uint8_t i=0; i<3; i++)
	{	
		_delay_ms(150);
		OE_L();
		WE_L();
		_delay_ms(150);
		OE_H();
		WE_H();	
	}
}


//az adatbuszra rak ki adatot
void put_data(uint8_t data)
{			
	OUTPUT_MODE();
	PORTC=(data&0b00111111);
	PORTB=((data&0b11000000)>>6);
//	INPUT_MODE();
}

//ez a fggvny rakja ki a cmbuszra az aktulis cmet
void put_address(uint16_t addr)
{	
    OUTPUT_MODE();
	put_data(addr);
	LATCH1_PULSE(); //az als 8 bit bersa
	put_data(addr>>8);
	LATCH2_PULSE(); //a fels 8 bit bersa
    INPUT_MODE();
}

//a buszrl olvas
uint8_t get_data(void)
{
    INPUT_MODE();
	uint8_t data=0;
	data=(PINB<<6)&0b11000000; //a fels 2 bit behozsa
	data|=PINC; //hozzrakni a maradk 6-ot
	return data;
}

//USART byte fogads
ISR(USART_RXC_vect) 
{
	cli();
	uint8_t rxchr=UDR;
	UDR=get_data();//rxchr; visszakldi
	sei();
}

//USART byte klds


//memria olvassa s kldse
void read_memory(void)
{
	uint16_t mem_addr=0;
	CE_L();
	_delay_ms(1);
	while(1)
	{
		OE_H();
		put_address(mem_addr);
		_delay_us(10);
		OE_L();
        _delay_us(10);
		ser_byte(get_data());
		mem_addr++;
		//amg vgig nem megy a memrin, vagy valamirt a PC-s progi meg nem szaktja az tvitelt
		if(mem_addr==mem_size || !(PIND&(1<<PD2)))
		{		
			CE_H();
			OE_H();
			return;
		}
	}
	
}

void write_memory(void)
{
	uint16_t mem_addr=0;
	OE_H();
	CE_L(); //kivlaszts
	WE_H();
	while(1)
	{
		put_address(mem_addr);
		while (!(UCSRA & (1<<RXC))); //amg nem rkezik adat, vrjon
		put_data(UDR);
		WE_L(); //cm latchels
		_delay_us(1);
		WE_H(); //adatlatchels
		mem_addr++;
		//amg tele nem lesz a memria, vagy valamirt a PC-s progi meg nem szaktja az tvitelt
		if(mem_addr==mem_size)// || !(PIND&(1<<PD2)))
		{		
			CE_H();
			WE_H();
			return;
		}
	}
	
}


int main(void)
{
	//inicializls
		///   76543210
	DDRD=0b11111010;   //CE,WE,OE,LP2,TXD,CTS=output, RXD, RTS input
	DDRB|=(1<<PB2); //LP1 output
	PORTB=0;
	PORTD=0;
	CE_H();
	OE_H();
	WE_H();
	
	//USART INIT
	UCSRB|=(1<<RXCIE)|(1<<RXEN)|(1<<TXEN); //megszakts be, bekapcsols
	UCSRC|=(1<<URSEL)|(1<<UCSZ0) | (1 << UCSZ1); // Use 8-bites csomagok, 1 stopbit
	UBRRL=3; 
	UBRRH=0; //57600baud
	
	startup_ledflash();
	OUTPUT_MODE();
	
	
	
	//kinullzni a buszokat
	put_data(0);
	LATCH1_PULSE();
	LATCH2_PULSE();
	
	uint8_t x=0;
	mem_size=0;
//	mem_addr=0;
   
	while(1)
	{
			
		//itt most vrni kell a parancsszra
		while ( !(UCSRA & (1<<RXC)) );
		usartchr=UDR;
		//UDR=usartchr; //visszajelzst ad a proginak
			
		//el kell dnteni hogy milyen feladat jtt
		switch(usartchr)
		{
			case 'A': mem_size=2048; break; //A memria mrett belltani
			case 'B': mem_size=8192; break;
			case 'C': mem_size=32768; break;
			//a tbbi ide jhet
			//.....
		//	case 'N': mem_addr=0; break;//visszalltani a cmet 0-ra
			case 'R':	read_memory();
						break;
			case 'W':	write_memory();
						break;
			default: break;
		}
	}

}


  //  CE_L();
  //  OE_L();
   // while(x<255)
   // {
    //    put_address(x);
//        _delay_us(1);
//        ser_decimal(get_data());
//        endl();
 //       x++;
 //       
  //  }
  //  OE_H();
    
//	CE_H();

//	WE_L();
//	while(1){};


	//	else 
	//	{
	//		WE_H(); // RTS=H akkor vilgt
	//		PORTD|=(1<<PD3);
	//	}

		//PORTD|=(1<<PD3);
		//OE_L();
		//_delay_ms(500);
		//PORTD&=~(1<<PD3);
		//OE_H();